/* This file will include set of functions
 * to store, retrieve and modify data from database.
 */

.import QtQuick.LocalStorage 2.0 as SQL


/* For internal usage in module.
 */
var gDbCache = undefined
function openStdDataBase() {
    if (gDbCache === undefined)
        gDbCache = SQL.LocalStorage.openDatabaseSync("RSS Reader", "1.0", "Feed DB", 10000)
    return gDbCache
}

function ensureFeedTableExists(transaction) {
    transaction.executeSql('CREATE TABLE IF NOT EXISTS UserFeeds(link STRING PRIMARY KEY, displayName STRING, siteLink STRING, description STRING)')
}

function ensureEntriesTableExists(transaction) {
    transaction.executeSql('CREATE TABLE IF NOT EXISTS FeedEntries(feedLink STRING, title STRING, pubDate STRING, hash STRING, description TEXT, content TEXT)')
}
/* Internal module usage end. */

/* This variable will store result of any operation with database.
 */
//var dbResult

/* Load all feed from database.
 * Mb later we will need some criteria, such as tag, date and so on!
 */
function loadFeeds() {
    var db = openStdDataBase()
    var dbResultLocal
    var feeds
    db.transaction(function(tx) {
        ensureFeedTableExists(tx)
        dbResultLocal = tx.executeSql("SELECT * FROM UserFeeds")
        console.log("SELECTED: ", dbResultLocal.rows.length)
        feeds = dbResultLocal.rows
    }
    )
    return feeds;
}

/* Add feed to database.
 * Now only 3 fields, later - more (date, tag and etc.)
 */
function addFeed(d) {
    if (d.link === undefined) {
        console.log("Database, AddFeed: 0.link can't be empty")
        return
    }
    var dbResult

    d.displayName = d.displayName || ""
    d.siteLink = d.siteLink || ""
    d.description = d.description || ""

    var db = openStdDataBase()

    db.transaction(function (tx) {
        ensureFeedTableExists(tx)

        /* Check uniqueness.
         */
        dbResult = tx.executeSql("SELECT * FROM UserFeeds WHERE link=?", [d.link])
        if (dbResult.rows.length > 0) {
            console.log("Database, addFeed: already exist feed with link", d.link)
            dbResult.exist = true // TODO TMP, mb need string with error, will see later.
            return
        }

        dbResult = tx.executeSql('INSERT INTO UserFeeds VALUES(?, ?, ?, ?)',
                                 [d.link , d.displayName, d.siteLink, d.description])
        console.log("INSERT ID", dbResult.insertId)
    }
    )
    return dbResult
}

/* Update information about feed with new information.
 */
function updateFeedWithInfo(link, d) {
    var db = openStdDataBase()
    var dbResult
    db.transaction(function (tx) {
        ensureFeedTableExists(tx)
        dbResult = tx.executeSql('UPDATE UserFeeds SET displayName=?, siteLink=?, description=? WHERE link=?',
                                 [d.displayName, d.siteLink, d.description, link])
        console.log("UPDATE, AFFECTED ROWS", dbResult.rowsAffected)
    }
    )
    return dbResult
}

/* Delete feed from database.
 */
function deleteFeed(link) {
    var db = openStdDataBase()
    var result
    db.transaction(function (tx) {
        var dbResultLocal
        ensureFeedTableExists(tx)
        dbResultLocal = tx.executeSql('DELETE FROM UserFeeds WHERE link = ?', [link])
        console.log("DELETE, AFFECTED ROWS", dbResultLocal.rowsAffected)
        result = dbResultLocal
    }
    )
    return result
}

/* Select all entries from database.
 */
function loadEntries(feedLink, all) {
    all = all || false
    var db = openStdDataBase()
    var entries;

    db.transaction(function(tx) {
        ensureEntriesTableExists(tx)
        var dbResultLocal
        if (all)
            dbResultLocal = tx.executeSql('SELECT * FROM FeedEntries')
        else dbResultLocal = tx.executeSql('SELECT * FROM FeedEntries WHERE feedLink = ?', [feedLink])
        console.log("loadEntries, SELECTED ROWS", dbResultLocal.rows.length)
        entries = dbResultLocal.rows;
    }
    )
    return entries;
}

/* Select all entries from database.
 */
function loadFeed(feedLink) {
    var db = openStdDataBase()
    var feed = {displayName: "Unknown"}
    db.transaction(function(tx) {
        ensureEntriesTableExists(tx)
        var dbResult = tx.executeSql('SELECT * FROM UserFeeds WHERE link = ?', [feedLink])
        console.log("feedName, SELECTED ROWS", dbResult.rows.length)
        if(dbResult.rows.length > 0) {
            feed = dbResult.rows.item(0)
        }
    }
    )
    return feed
}

/* Append entry to database.
 */
function addEntry(feedLink, hash, d) {
    var db = openStdDataBase()
    var dbResult
    if (feedLink === undefined || hash === undefined) {
        console.log("Database, addEntry: feedLink and hash can't be undefined")
        return
    }

    db.transaction(function(tx) {
        ensureEntriesTableExists(tx)

        /* Check uniqueness.
         */
        dbResult = tx.executeSql("SELECT * FROM FeedEntries WHERE hash=?", [hash])
        if (dbResult.rows.length > 0) {
            console.log("Database, addEntry: already exist entry with hash", hash)
            dbResult.exist = true
            return
        }

        dbResult = tx.executeSql('INSERT INTO FeedEntries VALUES(?, ?, ?, ?, ?, ?)',
                                 [feedLink, d.title, d.pubDate, hash, d.description, d.content])
        // console.log("ENTRY INSERT ID", dbResult.insertId)
    }
    )
    return dbResult
}

/* Remove all entries from database by feed link.
 */
function removeEntriesOfFeed(feedLink) {
    var db = openStdDataBase()
    var dbResult
    db.transaction(function(tx) {
        ensureEntriesTableExists(tx)
        dbResult = tx.executeSql('DELETE FROM FeedEntries WHERE feedLink = ?', [feedLink])
    }
    )
    return dbResult
}

/* Only for debug purposes.
 */
function dropTables() {
    var db = openStdDataBase()

    db.transaction(function(tx) {
        tx.executeSql("DROP TABLE IF EXISTS UserFeeds")
        tx.executeSql("DROP TABLE IF EXISTS FeedEntries")
        console.log("DATABASE DELETED")
    }
    )
}


